import sys
import rosbag
import numpy as np
import matplotlib.pyplot as plt

# Read data from bag
bag = rosbag.Bag(sys.argv[1])

pose_t = np.array([])
pose_x = np.array([])
pose_y = np.array([])
pose_theta = np.array([])
cmd_t = np.array([])
cmd_v = np.array([])
cmd_w = np.array([])
plan_t = np.array([])
plan_x = np.array([])
plan_y = np.array([])
plan_dx = np.array([])
plan_dy = np.array([])

for topic, msg, t in bag.read_messages():
    if topic == "/unicycle/pose":
        pose_t = np.append(pose_t, float(t.secs+t.nsecs*1.0e-9))
        pose_x = np.append(pose_x, msg.x)
        pose_y = np.append(pose_y, msg.y)
        pose_theta = np.append(pose_theta, msg.theta)

    if topic == "/unicycle/cmd_vel":
        cmd_t = np.append(cmd_t, float(t.secs+t.nsecs*1.0e-9))
        cmd_v = np.append(cmd_v, msg.linear.x)
        cmd_w = np.append(cmd_w, msg.angular.z)

    if topic == "/unicycle/desired_position":
        plan_t = np.append(plan_t, float(t.secs+t.nsecs*1.0e-9))
        plan_x = np.append(plan_x, msg.x)
        plan_y = np.append(plan_y, msg.y)

    if topic == "/unicycle/desired_velocity":
        plan_dx = np.append(plan_dx, msg.x)
        plan_dy = np.append(plan_dy, msg.y)

bag.close()

# Plot data
# Trajectory
plt.figure(1)

plt.subplot(3, 1, 1)
ref, = plt.plot(plan_t,plan_x,label='reference')
act, = plt.plot(pose_t,pose_x,label='actual')
plt.ylabel('x [m]')
plt.title('Trajectory')
plt.legend(handles=[ref, act])
plt.grid(True)

plt.subplot(3, 1, 2)
ref, = plt.plot(plan_t,plan_y,label='reference')
act, = plt.plot(pose_t,pose_y,label='actual')
plt.ylabel('y [m]')
plt.legend(handles=[ref, act])
plt.grid(True)

plt.subplot(3, 1, 3)
plt.plot(pose_t,pose_theta)
plt.xlabel('Time [s]')
plt.ylabel('theta [rad]')
plt.grid(True)

plt.figure(2)

ref, = plt.plot(plan_x,plan_y,label='reference')
act, = plt.plot(pose_x,pose_y,label='actual')
plt.xlabel('x [m]')
plt.ylabel('y [m]')
plt.legend(handles=[ref, act])
plt.title('Trajectory')
plt.grid(True)

# Robot commands
plt.figure(3)

plt.subplot(2, 1, 1)
plt.plot(cmd_t,cmd_v)
plt.ylabel('Linear velocity [m/s]')
plt.title('Robot commands')
plt.grid(True)

plt.subplot(2, 1, 2)
plt.plot(cmd_t,cmd_w)
plt.ylabel('Angular velocity [rad/s]')
plt.grid(True)

plt.show(block=False)

raw_input('Press enter to exit...')
plt.close()
exit()

